/* wptKeyCache.h
 *	Copyright (C) 2005, 2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */
#ifndef WPT_KEYCACHE_H
#define WPT_KEYCACHE_H

#include <gpgme.h>

struct gpg_desig_rev_s;
typedef struct gpg_desig_rev_s *gpg_desig_rev_t;

/* key cache specific update flags. */
enum keycache_flag_t {
    KC_FLAG_ADD = 1,
    KC_FLAG_UPD = 2
};

/* key specifc attributes. */
enum keycache_attr_t {
    KC_ATTR_PREFSYM	= 11,
    KC_ATTR_PREFKSERV	= 24
};


/* contains the UTF8 decoded user id. */
struct native_uid_s {
    struct native_uid_s *next;
    unsigned int malloced:1;	    /* 1 if the follow parts are actually
				       malloced or just referenced pointer. */
    unsigned int revoked : 1;
    gpgme_validity_t validity;

    gpgme_key_sig_t signatures;

    char *uid;
    char *name;
    char *comment;
    char *email;
};


struct keycache_s {
    struct keycache_s *next;
    int flags;
    gpgme_key_t key;			/* actual key context. */
    unsigned int default_key:1;		/* default signing key. */
    struct keycache_s *pubpart;		/* corresponding public key. */
    struct {
	unsigned int is_protected:1;	/* key is protected. */
	unsigned int divert_to_card:1;	/* secret key on card. */
	unsigned int has_desig_rev:1;	/* has desig. revokers. */
    } gloflags;
    struct {
	unsigned int   flags;
	unsigned char *d;	    /* attribute (JPG) data. */
	unsigned int   len;	    /* attribute data length. */
	gpgme_validity_t validity;  /* attribute id validity. */
    } attrib;
    unsigned char *sym_prefs;/* symmetric cipher preferences. */
    char *pref_keyserver;   /* if != NULL preferred keyserver. */
    unsigned short pref_keyserver_port;
    char *card_type;	    /* if != NULL key stored on smart card. */
    struct native_uid_s *uids;
    gpg_desig_rev_t rev;	/* if != NULL list of desig revokers. */
};

struct gpg_keycache_s {    
    struct keycache_s *item;
    struct keycache_s *tmp;
    int secret;
    int pos;
    void (*cb) (void*, const char*, int, int, int);
    void *cb_value;
    int cb_value2;
};
typedef struct gpg_keycache_s *gpg_keycache_t;

gpgme_error_t gpg_keycache_new (gpg_keycache_t * r_ctx);
void gpg_keycache_release (gpg_keycache_t ctx);
void gpg_keycache_item_release (struct keycache_s *c);
void gpg_keycache_set_cb (gpg_keycache_t ctx,
			    void (*cb)(void *, const char *, int, int, int),
			    void* cb_value1, int cb_value2);
void gpg_keycache_rewind (gpg_keycache_t ctx);
int gpg_keycache_get_size (gpg_keycache_t ctx);
gpgme_error_t gpg_keycache_fetch_key (const char *keyid, int is_sec, 
			gpgme_key_t *r_key, struct keycache_s **r_c);
gpgme_error_t gpg_keycache_add_key (gpg_keycache_t ctx, gpgme_key_t key, 
				      void **opaque);
gpgme_error_t gpg_keycache_update_key (gpg_keycache_t ctx, int is_sec, 
					 void *opaque, const char *keyid);
gpgme_error_t gpg_keycache_find_key (gpg_keycache_t ctx, const char *pattern,
				       int flags, gpgme_key_t * r_key);
gpgme_error_t gpg_keycache_find_key2 (gpg_keycache_t ctx, 
				      const char *pattern, int flags,
				        gpgme_key_t *r_key, 
					struct keycache_s **r_item);
gpgme_error_t gpg_keycache_delete_key (gpg_keycache_t ctx, 
					 const char * pattern);
gpgme_error_t gpg_keycache_init (gpg_keycache_t ctx, const char * pattern, 
				   int secret);
gpgme_error_t gpg_keycache_next_key (gpg_keycache_t ctx, int flags,
				       gpgme_key_t * r_key);
gpgme_error_t gpg_keycache_next_key2 (gpg_keycache_t ctx, int flags, 
				     struct keycache_s **c, gpgme_key_t *r_key);
gpgme_error_t gpg_keycache_prepare (gpg_keycache_t ctx, 
				      const char *pubring, 
				      const char *secring);
gpgme_error_t gpg_keycache_prepare_single (gpg_keycache_t ctx, 
					     const char *keyid,
					     const char *pubr, 
					     const char *secr);
gpgme_error_t gpg_keycache_sync (gpg_keycache_t pub, gpg_keycache_t sec);
gpgme_error_t gpg_keycache_next_updated_key (gpg_keycache_t ctx,
					     struct keycache_s **r_obj,
					     int *r_status);
gpgme_error_t gpg_keycache_set_default_key (gpg_keycache_t ctx, 
					    const char *pattern);
gpgme_error_t gpg_keycache_get_default_key (gpg_keycache_t ctx,
					    gpgme_key_t *r_key);
gpgme_error_t gpg_keycache_update_attr (struct keycache_s *item,
					int attr, int force);

#endif /*WPT_KEYCACHE_H*/
