/* wptKeyManager.h - Key manager
 *	Copyright (C) 2001-2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#ifndef WPT_KEYMANAGER_H
#define WPT_KEYMANAGER_H

#include "wptContext.h"

#define send_cmd_id(hwnd, id) \
    PostMessage ((hwnd), WM_COMMAND, MAKEWPARAM ((id), 0), 0)

#define KM_PRELOAD_KEYS 6

/* possible key flags. */
enum km_flag_t {
    KM_FLAG_REVOKED  = 0x01,
    KM_FLAG_EXPIRED  = 0x02,
    KM_FLAG_DISABLED = 0x04,
};


enum km_ks_t {
    KM_KS_REFRESH = 0x01
};

struct km_group_s {
    config_file_t   gh;
    HWND	    tree;
    unsigned int    need_sync:1;
    char	*cfgfile;
};
typedef struct km_group_s * km_group_t;

struct km_group_cb_s {
    km_group_s *gc;
    char        name[256];
    int		use_name;
};


/*-- wptClipImportDlg.cpp --*/
int print_import_status (gpgme_import_result_t res);

/*-- wptKeyManager.cpp --*/
int km_check_for_seckey (listview_ctrl_t lv, int pos, int * utrust);
int km_check_key_status( listview_ctrl_t lv, int pos );
int km_get_key_status( listview_ctrl_t lv, int pos );
char* km_key_get_info (winpt_key_t key, int is_sec);
int km_clip_export( HWND dlg, listview_ctrl_t lv);
int km_file_export( HWND dlg, listview_ctrl_t lv, const char *fname );
int km_privkey_export(HWND dlg, listview_ctrl_t lv, const char *fname );
int km_set_implicit_trust (HWND dlg, listview_ctrl_t lv, int pos);
int km_clip_import (HWND dlg, int *r_newkeys, int *r_newsks);
int km_file_import (HWND dlg, const char *fname, 
		    int *r_newkeys, int *r_newsks);
int km_http_import (HWND dlg, const char * url);
int km_dropped_file_import (HWND dlg, HDROP hdrop, 
			    int *r_newkeys, int *r_newsks);
int km_delete_keys( listview_ctrl_t lv, HWND dlg );
int km_send_to_keyserver( listview_ctrl_t lv, HWND dlg, const char * host, unsigned short port);
int km_send_to_mail_recipient( listview_ctrl_t lv, HWND dlg );
int km_refresh_from_keyserver(listview_ctrl_t lv, HWND dlg);
int km_check_if_protected( listview_ctrl_t lv, int pos );
int km_enable_disable_key( listview_ctrl_t lv, HWND dlg, int pos, int enable );
void km_set_clip_info( const char *uid );
int km_key_is_v3( listview_ctrl_t lv, int pos );
void km_find_key (HWND dlg, listview_ctrl_t lv);
char* km_gen_export_filename (const char *keyid, int is_secret);
int km_get_key (listview_ctrl_t lv, int pos, winpt_key_t k);
gpgme_key_t km_get_key_ptr (listview_ctrl_t lv, int idx, 
			    struct keycache_s **r_ctx);
void km_key_show_revoc_info (winpt_key_t k);

#endif /* WPT_KEYMANAGER_H */
